/**
 * 
 * Copyright (c) 2009 Emanuil Rusev - http://erusev.com
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * The software is provided "as is", without warranty of any kind,
 * express or implied, including but not limited to the warranties
 * of merchantability, fitness for a particular purpose and
 * noninfringement. In no event shall the authors or copyright
 * holders be liable for any claim, damages or other liability,
 * whether in an action of contract, tort or otherwise, arising
 * from, out of or in connection with the software or the use or
 * other dealings in the software.
 *
 */

package com.erusev.control
{
	import com.erusev.control.a2.PauseButton;
	import com.erusev.control.a2.PlayButton;
	import com.erusev.control.a2.PlayheadController;
	import com.erusev.control.a2.VolumeButton;
	import com.erusev.core.Component;
	import com.erusev.media.Sound_;
	import com.erusev.util.TimeUtil;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	public class A2 extends Component
	{
		public function A2()
		{
			super();
			
			setCredits("MP3 Player by erusev.com", "http://a2.erusev.com");
			
			sound = new Sound_();
			sound.volume = .1
			sound.addEventListener("complete", sound_onComplete);
			sound.addEventListener("loadProgress", sound_onLoadProgress);
			sound.addEventListener("play", sound_onPlay);
			sound.addEventListener("playheadUpdate", sound_onPlayheadUpdate);
			sound.addEventListener("stop", sound_onStop);
			sound.addEventListener("volumeChange", sound_onVolumeChange);
			
			textFormat = new TextFormat();
			textFormat.font = "Verdana";
			textFormat.size = 12;
		}
		
		////////////////////////////////////////
		// objects
		////////////////////////////////////////
		
		private var sound:Sound_;
		private var textFormat:TextFormat;
		
		////////////////////////////////////////
		// properties
		////////////////////////////////////////
		
		// trackURL
		////////////////////////////////////////
		
		private var _trackURL:String = "";
		private var trackURLChanged:Boolean = true;
		
		public function get trackURL():String
		{
			return _trackURL;
		}
		
		public function set trackURL(value:String):void
		{
			_trackURL = value;
			trackURLChanged = true;
			
			invalidateProperties();
		}
		
		// volume
		////////////////////////////////////////
		
		private var _volume:int = 2;
		private var volumeChanged:Boolean = true;
		
		public function get volume():int
		{
			return _volume;
		}
		
		public function set volume(value:int):void
		{
			value = Math.max(value, 0);
			value = Math.min(value, 3);
			
			_volume = value;
			volumeChanged = true;
			
			invalidateProperties();
		}
		
		// color1
		////////////////////////////////////////
		
		private var _color1:uint = 0x777777;
		private var color1Changed:Boolean = true;
		
		public function get color1():uint
		{
			return _color1;
		}
		
		public function set color1(value:uint):void
		{
			_color1 = value;
			color1Changed = true;
			
			invalidateProperties();
		}
		
		// color2
		////////////////////////////////////////
		
		private var _color2:uint = 0x3377DD;
		private var color2Changed:Boolean = true;
		
		public function get color2():uint
		{
			return _color2;
		}
		
		public function set color2(value:uint):void
		{
			_color2 = value;
			color2Changed = true;
			
			invalidateProperties();
		}
		
		// autoPlay
		
		private var _autoPlay:Boolean;
		private var autoPlayChanged:Boolean;
		
		public function get autoPlay():Boolean
		{
			return _autoPlay;
		}
		
		public function set autoPlay(value:Boolean):void
		{
			_autoPlay = value;
			autoPlayChanged = true;
			
			invalidateProperties();
		}
		
		// loop
		
		private var _loop:Boolean;
		
		public function set loop(value:Boolean):void
		{
			_loop = value;
		}
		
		// playing (read-only)
		
		public function get playing():Boolean
		{
			return sound.playing;
		}
		
		////////////////////////////////////////
		// public methods
		////////////////////////////////////////
		
		public function play():void
		{
			sound.play();
		}
		
		////////////////////////////////////////
		// display objects
		////////////////////////////////////////
		
		private var playButton:PlayButton;
		private var pauseButton:PauseButton;
		private var track:PlayheadController;
		private var textField:TextField;
		private var volumeButton:VolumeButton;
		
		////////////////////////////////////////
		// overridden methods
		////////////////////////////////////////
		
		override protected function createChildren():void
		{
			super.createChildren();
			
			playButton = new PlayButton();
			playButton.addEventListener(MouseEvent.CLICK, playButton_onClick);
			addChild(playButton);
			
			pauseButton = new PauseButton();
			pauseButton.addEventListener(MouseEvent.CLICK, pauseButton_onClick);
			pauseButton.visible = false;
			addChild(pauseButton);
			
			volumeButton = new VolumeButton();
			volumeButton.color = 0x44ee99;
			volumeButton.addEventListener(MouseEvent.CLICK, volumeButton_onClick);
			addChild(volumeButton);
			
			track = new PlayheadController();
			track.addEventListener(MouseEvent.MOUSE_DOWN, track_onMouseDown);
			track.addEventListener(MouseEvent.MOUSE_UP, track_onMouseUp);
			track.addEventListener(MouseEvent.MOUSE_MOVE, track_onMouseMove);
			track.addEventListener(MouseEvent.MOUSE_OUT, track_onMouseOut);
			addChild(track);
			
			textField = new TextField();
			textField.autoSize = TextFieldAutoSize.LEFT;
			textField.selectable = false;
			textField.defaultTextFormat = textFormat;
			textField.mouseEnabled = false;
			textField.text = "00:00";
			addChild(textField);
		}
		
		override protected function commitProperties():void
		{
			super.commitProperties();
			
			if (trackURLChanged)
			{
				trackURLChanged = false;
				
				sound.url = _trackURL;
				
				if (_autoPlay)
				{
					sound.play();
				}
				
				track.loadProgress = 0;
			}
			
			if (volumeChanged)
			{
				volumeChanged = false;
				
				sound.volume = _volume/3
			}
			
			if (color1Changed)
			{
				color1Changed = false;
				
				playButton.color = _color1;
				pauseButton.color = _color1;
				track.color = _color1;
				volumeButton.color = _color1;
				
				// text
				
				textFormat.color = _color1;
				textField.defaultTextFormat = textFormat;
				textField.setTextFormat(textFormat);
			}
			
			if (color2Changed)
			{
				color2Changed = false;
				
				track.playheadColor = _color2;
			}
		}
		
		override protected function measure():void
		{
			super.measure();
			
			measuredWidth = 300;
			measuredHeight = 21;
		}
		
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			volumeButton.x = width - volumeButton.width;
			
			textField.x = width - textField.width - volumeButton.width - 5;
			textField.y = Math.floor((21 - textField.height)/2);
			
			track.x = playButton.width + 5;
			track.y = 5;
			track.width = width - track.x - textField.width - volumeButton.width - 10;
			track.height = 11;
		}
		
		////////////////////////////////////////
		// state
		////////////////////////////////////////
		
		private var dragging:Boolean;
		
		////////////////////////////////////////
		// event handlers
		////////////////////////////////////////
		
		private function track_onMouseDown(e:MouseEvent):void
		{
			dragging = true;
			
			textFormat.italic = true;
			textField.defaultTextFormat = textFormat;
			textField.text = TimeUtil.formatTime(e.localX/track.width*sound.estimatedDuration);
		}
		
		private function track_onMouseUp(e:MouseEvent):void
		{
			if (dragging)
			{
				sound.playheadPosition = (e.localX/track.width)*sound.estimatedDuration;
				
				if (sound.playing) sound.play();
				
				dragging = false;
				
				textFormat.italic = false;
				textField.defaultTextFormat = textFormat;
				textField.text = TimeUtil.formatTime(sound.elapsedPortion*sound.estimatedDuration);
			}
		}
		
		private function track_onMouseOut(e:MouseEvent):void
		{
			dragging = false;
			
			textFormat.italic = false;
			textField.defaultTextFormat = textFormat;
			textField.text = TimeUtil.formatTime(sound.elapsedPortion*sound.estimatedDuration);
		}
		
		private function track_onMouseMove(e:MouseEvent):void
		{
			if (dragging)
			{
				textFormat.italic = true;
				textField.defaultTextFormat = textFormat;
				textField.text = TimeUtil.formatTime(e.localX/track.width*sound.estimatedDuration);
			}
		}
		
		// sound
		
		private function sound_onPlay(e:Event):void
		{
			playButton.visible = false;
			pauseButton.visible = true;
		}
		
		private function sound_onStop(e:Event):void
		{
			playButton.visible = true;
			pauseButton.visible = false;
		}
		
		private function sound_onLoadProgress(e:Event):void
		{
			track.loadProgress = sound.loadedPortion;
		}
		
		private function sound_onPlayheadUpdate(e:Event):void
		{
			track.playheadProgress = sound.elapsedPortion;
			
			if (!dragging)
			{
				textFormat.italic = false;
				textField.defaultTextFormat = textFormat;
				textField.text = TimeUtil.formatTime(sound.elapsedPortion*sound.estimatedDuration);
			}
		}
		
		private function sound_onComplete(e:Event):void
		{
			sound.playheadPosition = 0;
			
			if (_loop)
			{
				sound.play();
			}
		}
		
		private function sound_onVolumeChange(e:Event):void
		{
			volumeButton.mode = _volume;
		}
		
		private function playButton_onClick(e:MouseEvent):void
		{
			sound.play();
		}
		
		private function pauseButton_onClick(e:MouseEvent):void
		{
			sound.stop();
		}
		
		private function volumeButton_onClick(e:MouseEvent):void
		{
			volume = (_volume + 1)%4;
		}
	}
}